var WebBrowser = require('expo-web-browser');
var reactNative = require('@magic-sdk/react-native');
var sha256Fallback = require('crypto-js/sha256');
var Base64 = require('crypto-js/enc-base64');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () {
            return e[k];
          }
        });
      }
    });
  }
  n['default'] = e;
  return n;
}

var WebBrowser__namespace = /*#__PURE__*/_interopNamespace(WebBrowser);
var sha256Fallback__default = /*#__PURE__*/_interopDefaultLegacy(sha256Fallback);
var Base64__default = /*#__PURE__*/_interopDefaultLegacy(Base64);

const CHARSET = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-._~';
const HAS_CRYPTO = typeof window !== 'undefined' && !!window.crypto;
const HAS_SUBTLE_CRYPTO = HAS_CRYPTO && !!window.crypto.subtle;
/**
 * Stringifies `bytes` using the OAuth 2.0 `code_verifier` character set.
 */

function bytesToVerifierString(bytes) {
  return Array.from(bytes).map(value => CHARSET[value % CHARSET.length]).join('');
}

function base64URLEncodeFromByteArray(arg) {
  const makeURLSafe = base64 => {
    return base64.replace(/\+/g, '-').replace(/\//g, '_').replace(/=/g, '');
  };

  if (arg instanceof ArrayBuffer) {
    const bytes = new Uint8Array(arg);
    const utf8Binary = Array.from(bytes).map(value => String.fromCharCode(value)).join('');
    const base64 = btoa(utf8Binary);
    return makeURLSafe(base64);
  }

  return makeURLSafe(Base64__default['default'].stringify(arg));
}
/**
 * Produces a SHA-256 hash of the given `message`. This function first attempts
 * to use the browser's built-in `SubtleCrypto` API, falling back to
 * CryptoJS if required.
 */


async function sha256(message) {
  if (HAS_SUBTLE_CRYPTO) {
    const bytes = new TextEncoder().encode(message);
    return crypto.subtle.digest('SHA-256', bytes).then(base64URLEncodeFromByteArray);
  }

  return base64URLEncodeFromByteArray(sha256Fallback__default['default'](message));
}
/**
 * Creates a cryptographically random string using the browser's built-in
 * `Crypto` API, falling back to `Math.random` if required.
 */


function createRandomString(size) {
  const bytes = new Uint8Array(size);

  if (HAS_CRYPTO) {
    window.crypto.getRandomValues(bytes);
  } else {
    for (let i = 0; i < size; i += 1) bytes[i] = Math.floor(Math.random() * Math.floor(255));
  }

  return bytesToVerifierString(bytes);
}
/**
 * Creates OAuth 2.0-compatible `code_verifier`, `code_challenge`, and `state`
 * parameters.
 */


async function createCryptoChallenge() {
  const state = createRandomString(128);
  const verifier = createRandomString(128);
  const challenge = await sha256(verifier);
  return {
    verifier,
    challenge,
    state
  };
}

exports.OAuthPayloadMethods = void 0;

(function (OAuthPayloadMethods) {
  OAuthPayloadMethods["ParseRedirectResult"] = "magic_oauth_parse_redirect_result";
})(exports.OAuthPayloadMethods || (exports.OAuthPayloadMethods = {}));

exports.OAuthErrorCode = void 0;

(function (OAuthErrorCode) {
  OAuthErrorCode["InvalidRequest"] = "invalid_request";
  OAuthErrorCode["InvalidClient"] = "invalid_client";
  OAuthErrorCode["InvalidScope"] = "invalid_scope";
  OAuthErrorCode["InvalidGrant"] = "invalid_grant";
  OAuthErrorCode["UnauthorizedClient"] = "unauthorized_client";
  OAuthErrorCode["UnsupportedResponseType"] = "unsupported_response_type";
  OAuthErrorCode["UnsupportedGrantType"] = "unsupported_grant_type";
  OAuthErrorCode["UnsupportedTokenType"] = "unsupported_token_type";
  OAuthErrorCode["AccessDenied"] = "access_denied";
  OAuthErrorCode["ServerError"] = "server_error";
  OAuthErrorCode["TemporarilyUnavailable"] = "temporarily_unavailable";
})(exports.OAuthErrorCode || (exports.OAuthErrorCode = {}));

class OAuthExtension extends reactNative.Extension.Internal {
  constructor(...args) {
    super(...args);
    this.name = 'oauth';
    this.config = {};
    this.compat = {
      'magic-sdk': false,
      '@magic-sdk/react-native': '>=2.7.0'
    };
  }

  loginWithPopup(configuration) {
    return this.utils.createPromiEvent(async (resolve, reject) => {
      try {
        const {
          provider,
          query,
          redirectURI
        } = await createURI.call(this, configuration);
        const url = `https://auth.magic.link/v1/oauth2/${provider}/start?${query}`;
        /**
         * Response Type
         * https://docs.expo.io/versions/latest/sdk/webbrowser/#returns
         */

        const res = await WebBrowser__namespace.openAuthSessionAsync(url, redirectURI);

        if (res.type === 'success') {
          const queryString = new URL(res.url).search;
          resolve(getResult.call(this, queryString.toString()));
        } else {
          reject(this.createError(res.type, 'User has cancelled the authentication', {}));
        }
      } catch (err) {
        reject(this.createError(err.message, 'An error has occurred', {
          err
        }));
      }
    });
  }

}
const OAUTH_REDIRECT_METADATA_KEY = 'oauth_redirect_metadata';
async function createURI(configuration) {
  // Bust any old, in-progress OAuth flows.
  await this.utils.storage.removeItem(OAUTH_REDIRECT_METADATA_KEY); // Unpack configuration, generate crypto values, and persist to storage.

  const {
    provider,
    redirectURI,
    scope,
    loginHint
  } = configuration;
  const {
    verifier,
    challenge,
    state
  } = await createCryptoChallenge();
  /* Stringify for RN Async storage */

  const storedData = JSON.stringify({
    verifier,
    state
  });
  await this.utils.storage.setItem(OAUTH_REDIRECT_METADATA_KEY, storedData); // Formulate the initial redirect query to Magic's OAuth hub.
  // Required fields:
  //   - `magic_api_key`
  //   - `magic_challenge`
  //   - `state`
  //   - `redirect_uri`
  //   - `platform`

  const query = [`magic_api_key=${encodeURIComponent(this.sdk.apiKey)}`, `magic_challenge=${encodeURIComponent(challenge)}`, `state=${encodeURIComponent(state)}`, `platform=${encodeURIComponent('rn')}`, scope && `scope=${encodeURIComponent(scope.join(' '))}`, redirectURI && `redirect_uri=${encodeURIComponent(redirectURI)}`, loginHint && `login_hint=${encodeURIComponent(loginHint)}`].reduce((prev, next) => next ? `${prev}&${next}` : prev);
  return {
    query,
    provider,
    redirectURI
  };
}
function getResult(queryString) {
  return this.utils.createPromiEvent(async (resolve, reject) => {
    const json = await this.utils.storage.getItem(OAUTH_REDIRECT_METADATA_KEY);
    const {
      verifier,
      state
    } = JSON.parse(json); // Remove the save OAuth state from storage, it stays in memory now...

    this.utils.storage.removeItem(OAUTH_REDIRECT_METADATA_KEY);
    const parseRedirectResult = this.utils.createJsonRpcRequestPayload(exports.OAuthPayloadMethods.ParseRedirectResult, [queryString, verifier, state]); // Parse the result, which may contain an OAuth-formatted error.

    const resultOrError = await this.request(parseRedirectResult);
    const maybeResult = resultOrError;
    const maybeError = resultOrError;

    if (maybeError.error) {
      var _maybeError$error_des;

      reject(this.createError(maybeError.error, (_maybeError$error_des = maybeError.error_description) != null ? _maybeError$error_des : 'An error occurred.', {
        errorURI: maybeError.error_uri,
        provider: maybeError.provider
      }));
    }

    resolve(maybeResult);
  });
}

exports.OAuthExtension = OAuthExtension;
exports.createURI = createURI;
exports.getResult = getResult;
//# sourceMappingURL=index.js.map
